@@ -315,13 +315,13 @@ def swap_collection(fn: Callable[..., Any], col_a: str, col_b: str):
 @dataclasses.dataclass(frozen=True)
 class In(Generic[T]):
   """Specifies a variable collection should only be lifted as input."""
-  axis: Any  # pytype does not support generic variable annotation
+  axis: T
 
 
 @dataclasses.dataclass(frozen=True)
 class Out(Generic[T]):
   """Specifies a variable collection should only be lifted as output."""
-  axis: Any  # pytype does not support generic variable annotation
+  axis: T
 
 
 def _split_in_out_axes(xs: Mapping[CollectionFilter, Any]):
@@ -336,9 +336,9 @@ InOutAxis = Union[Axis, In[Axis], Out[Axis]]
 
 
 def _bwd_wrapper(treedef, bwd_fn, tangent):
-  vars_grad, inputs_grad = bwd_fn(tangent)
+  vars_grad, *inputs_grad = bwd_fn(tangent)
   vars_grad = treedef.unflatten(vars_grad)
-  return inputs_grad, vars_grad
+  return (vars_grad, *inputs_grad)
 
 
 def vjp(
@@ -362,13 +362,13 @@ def vjp(
 
   Example::
 
-    def learn_scale(scope, x):
+    def learn_scale(scope, x, y):
       p = scope.param('scale', nn.initializers.zeros, ())
-      return p * x
-    def f(scope, x):
-      y, bwd = lift.vjp(learn_scale, scope, x)
-      params_grad, x_grad = bwd(jnp.ones(y.shape))
-      return y, params_grad, x_grad
+      return p * x * y
+    def f(scope, x, y):
+      z, bwd = lift.vjp(learn_scale, scope, x, y)
+      params_grad, x_grad, y_grad = bwd(jnp.ones(z.shape))
+      return z, params_grad, x_grad, y_grad
 
   Args:
     fn: Function to be differentiated. Its arguments should be arrays, scalars,
@@ -1049,7 +1049,7 @@ def custom_vjp(fn: Callable[..., Any],
   passed to `backward_fn`.
 
   The `backward_fn` receives the nondiff arguments, residuals, and the output
-  tangents. It should return a tuple containing the input and variable tangents.
+  tangents. It should return a tuple containing the variable and input tangents.
 
   Note that the vjp function returned by `lift.vjp` can be passed as residual
   and used in the `backward_fn`. The scope is unavailable during the backward
@@ -1065,9 +1065,9 @@ def custom_vjp(fn: Callable[..., Any],
       return y, vjp_fn
 
     def bwd(features, vjp_fn, y_t):
-      input_t, params_t = vjp_fn(y_t)
+      params_t, *inputs_t = vjp_fn(y_t)
       params_t = jax.tree_util.tree_map(jnp.sign, params_t)
-      return input_t, params_t
+      return (params_t, *inputs_t)
 
     dense_sign_grad = lift.custom_vjp(
         f, forward_fn=fwd, backward_fn=bwd, nondiff_argnums=(2,))
@@ -1080,8 +1080,8 @@ def custom_vjp(fn: Callable[..., Any],
       `backward_fn`.
     backward_fn: arguments are passed as (*nondiff_args, residuals, tangents)
       The function should return a tuple containing the tangents for the
-      input arguments (except the scope and nondiff args) and the variable
-      tangents for the collections specified by `grad_vars`.
+      variable in the collections specified by `grad_vars` and the input
+      arguments (except the scope and nondiff args).
     grad_vars: The collections for which a vjp will be computed
       (default: "params").
     nondiff_argnums: arguments for which no vjp is computed.
@@ -1116,10 +1116,10 @@ def custom_vjp(fn: Callable[..., Any],
       nondiff_args = args[:-2]
       res, g = args[-2:]  # pylint: disable=unbalanced-tuple-unpacking
       g_y, _ = g
-      input_t, var_t = backward_fn(*nondiff_args, res, g_y)
+      var_t, *inputs_t = backward_fn(*nondiff_args, res, g_y)
       assert scopes_treedef is not None, 'backward called before forward?!'
       var_t = tuple(scopes_treedef.flatten_up_to(var_t))
-      return var_t, input_t
+      return (var_t, *inputs_t)
 
     f.defvjp(f_fwd, f_bwd)
 
@@ -1138,6 +1138,7 @@ def checkpoint(fn: Callable[..., Any],
                rngs: PRNGSequenceFilter = True,
                concrete: bool = False,
                prevent_cse: bool = True,
+               static_argnums: Union[int, Tuple[int, ...]] = (),
                policy: Optional[Callable[..., bool]] = None,
                ) -> Callable[..., Any]:
   """Lifted version of ``jax.checkpoint``.
@@ -1164,15 +1165,21 @@ def checkpoint(fn: Callable[..., Any],
       ``pmap``, CSE can defeat the purpose of this decorator. But in some
       settings, like when used inside a ``scan``, this CSE prevention mechanism
       is unnecessary, in which case ``prevent_cse`` can be set to False.
+    static_argnums: Optional, int or sequence of ints, indicates which argument
+      values on which to specialize for tracing and caching purposes. Specifying
+      arguments as static can avoid ConcretizationTypeErrors when tracing, but
+      at the cost of more retracing overheads.
     policy: Experimental checkpoint policy, see ``jax.checkpoint``.
   Returns:
     A wrapped version of ``fn``. When computing gradients intermediate
     computations will be re-computed when computing gradients.
   """
   def inner(scope_fn, repack_fn, variable_groups, rng_groups, *args, **kwargs):
+    # add 2 to each static_argnums because we add two initial arguments to rematted
+    static_argnums_ = jax.tree_util.tree_map(lambda x: x + 2, static_argnums)
     @functools.partial(jax.remat,
-                       concrete=concrete, prevent_cse=prevent_cse,
-                       policy=policy)
+                       concrete=concrete, static_argnums=static_argnums_,
+                       prevent_cse=prevent_cse, policy=policy)
     @functools.wraps(fn)
     def rematted(variable_groups, rng_groups, *args, **kwargs):
       scope = scope_fn(variable_groups, rng_groups)
