@@ -22,7 +22,7 @@ import typing
 from typing import (Any, Callable, Dict, Generic, Iterable, Mapping, Optional,
                     Sequence, Set, Tuple, TypeVar, Union)
 
-from . import tracers
+from flax.ids import uuid
 from flax import config
 from flax import errors
 from flax import struct
@@ -30,6 +30,7 @@ from flax import traceback_util
 from .frozen_dict import freeze
 from .frozen_dict import FrozenDict
 from .frozen_dict import unfreeze
+from . import tracers
 import jax
 from jax import config as jax_config
 from jax import numpy as jnp
@@ -51,7 +52,6 @@ Filter = Union[bool, str, typing.Collection[str], 'DenyList']
 # When conditioning on filters we require explicit boolean comparisons.
 # pylint: disable=g-bool-id-comparison
 
-
 @dataclasses.dataclass(frozen=True, eq=True)
 class DenyList:
   """DenyList represents an opt-out based mutability filter.
@@ -319,7 +319,7 @@ def group_collections(
     group = {}
     for col in cols:
       if in_filter(col_filter, col):
-        group[col] = jax.tree_map(lambda x: x, xs[col])
+        group[col] = jax.tree_util.tree_map(lambda x: x, xs[col])
       else:
         remaining_cols.append(col)
     cols = remaining_cols
@@ -343,6 +343,7 @@ class Variable(Generic[T]):
       collection: The collection of the variable (e.g., "params").
       name: The name of the variable (e.g., "dense").
     """
+    self._id = uuid()
     self.scope = scope
     self.collection = collection
     self.name = name
@@ -368,6 +369,12 @@ class _ChildRNGSentinel:
 child_rng_token = _ChildRNGSentinel()
 
 
+class _DefaultSentinel:
+  pass
+# used to denote no default flag value on scope
+no_flag = _DefaultSentinel()
+
+
 class Scope:
   """A Scope allows easy access to variables and manages RNGS of a neural network layer.
 
@@ -387,7 +394,8 @@ class Scope:
                name: Optional[str] = None,
                mutable: CollectionFilter = False,
                parent: Optional['Scope'] = None,
-               path: Iterable[str] = ()):
+               path: Iterable[str] = (),
+               flags: Optional[Dict] = None):
     """Initializes a Scope.
 
     Args:
@@ -397,6 +405,7 @@ class Scope:
       mutable: A CollectionFilter determining which variables are mutable.
       parent: The parent scope.
       path: The path in the variable tree from the root scope to this scope.
+      flags: internal flags.
     """
     rngs = {k: LazyRng.create(v) for k, v in rngs.items()} if rngs else {}
     self._variables = variables
@@ -405,6 +414,7 @@ class Scope:
     self.path = tuple(path)
     self.rngs = rngs
     self.mutable = mutable
+    self.flags = freeze({} if flags is None else flags)
 
     self._root = parent.root if parent else None
     self.trace_level = tracers.trace_level(tracers.current_trace())
@@ -485,7 +495,7 @@ class Scope:
     """
     self._check_valid()
     scope = Scope(self._variables, self.rngs, self.name, self.mutable,
-                  self.parent)
+                  self.parent, path=self.path, flags=self.flags)
     if not rewind_rngs:
       scope.rng_counters = self.rng_counters
     return scope
@@ -552,7 +562,8 @@ class Scope:
                   rngs=rngs,
                   parent=self,
                   mutable=self.mutable,
-                  path=self.path + (name,))
+                  path=self.path + (name,),
+                  flags=self.flags)
     scope.rng_counters = rng_counters
     return scope
 
@@ -570,7 +581,7 @@ class Scope:
       fn: the function to partially apply the child Scope to.
       name: optional name of the child.
       prefix: prefix used for generating name if it is `None`.
-      named_call: if true, `fn` will be wrapped with `lift.named_call`. The XLA
+      named_call: if true, `fn` will be run under `jax.named_scope`. The XLA
         profiler will use this to name tag the computation.
       **partial_kwargs: additional kwargs partially applied to `fn`.
 
@@ -582,15 +593,16 @@ class Scope:
         prefix = fn.__name__ + '_' if hasattr(fn, '__name__') else ''
       name = self.default_name(prefix)
     scope = self.push(name)
-    if named_call:
-      # We import named_call at runtime to avoid a circular import issue.
-      from . import lift  # pylint: disable=g-import-not-at-top
-      fn = lift.named_call(fn, name)
 
     @functools.wraps(fn)
     def wrapper(*args, **kwargs):
       kwargs = dict(partial_kwargs, **kwargs)
-      return fn(scope.rewound(), *args, **kwargs)
+      if named_call:
+        with jax.named_scope(name):
+          res = fn(scope.rewound(), *args, **kwargs)
+      else:
+        res = fn(scope.rewound(), *args, **kwargs)
+      return res
 
     return wrapper
 
@@ -684,7 +696,16 @@ class Scope:
     if not self.is_mutable_collection(col):
       raise errors.ModifyScopeVariableError(col, name, self.path_text)
     variables = self._mutable_collection(col)
-    variables[name] = value
+    # Make sure reference sharing of child variable dictionaries isn't broken
+    def put(target, key, val):
+      if (key in target and isinstance(target[key], dict) and
+          isinstance(val, Mapping)):
+        for k, v in val.items():
+          put(target[key], k, v)
+      else:
+        target[key] = val
+
+    put(variables, name, value)
 
   def variable(self, col: str, name: str,  # pylint: disable=keyword-arg-before-vararg
                init_fn: Optional[Callable[..., T]] = None,
@@ -737,8 +758,8 @@ class Scope:
       # catch it with an error message.
       # NOTE: We could consider moving this to `self.`
       abs_value = jax.eval_shape(lambda rng: init_fn(rng, *init_args), abs_rng)
-      abs_value_flat = jax.tree_leaves(abs_value)
-      value_flat = jax.tree_leaves(value)
+      abs_value_flat = jax.tree_util.tree_leaves(abs_value)
+      value_flat = jax.tree_util.tree_leaves(value)
       for val, abs_val in zip(value_flat, abs_value_flat):
         # NOTE: We could check dtype consistency here as well but it's
         # usefuleness is less obvious. We might intentionally change the dtype
@@ -761,6 +782,14 @@ class Scope:
     for col in collections:
       self._collection(col)
 
+  def has_flag(self, key) -> bool:
+    return key in self.flags
+
+  def get_flag(self, key, default=no_flag) -> Any:
+    if key not in self.flags and default is no_flag:
+      return ValueError(f'Flag {key} not present on scope.')
+    return self.flags.get(key, default)
+
 
 def _unfreeze_variables(variables, mutable):
   new_variables = {}
@@ -774,7 +803,8 @@ def _unfreeze_variables(variables, mutable):
 
 def bind(variables: VariableDict,
          rngs: Optional[RNGSequences] = None,
-         mutable: CollectionFilter = False):
+         mutable: CollectionFilter = False,
+         flags: Optional[Dict] = None):
   """Binds variables and rngs to a new ``Scope``.
 
   bind provides a ``Scope`` instance without transforming a function with
@@ -791,6 +821,7 @@ def bind(variables: VariableDict,
     variables: Variable dictionary to bind.
     rngs: RNGs to bind.
     mutable: Which variable colections to treat as mutable.
+    flags: internal flags.
 
   Returns:
     A new scope with the variables and rngs bound to it.
@@ -801,16 +832,18 @@ def bind(variables: VariableDict,
     raise errors.InvalidRngError(
         'rngs should be a dictionary mapping strings to `jax.PRNGKey`.')
   new_variables = _unfreeze_variables(variables, mutable)
-  return Scope(new_variables, rngs=rngs, mutable=mutable)
+  return Scope(new_variables, rngs=rngs, mutable=mutable, flags=flags)
 
 
 def apply(fn: Callable[..., Any],
-          mutable: CollectionFilter = False) -> Callable[..., Any]:
+          mutable: CollectionFilter = False,
+          flags: Optional[Dict] = None) -> Callable[..., Any]:
   """Functionalize a `Scope` function.
 
   Args:
     fn: a function taking a `Scope` as its first argument.
     mutable: the filter determining which variable collections are mutable.
+    flags: internal flags.
 
   Returns:
     `fn` with the scope partially applied.
@@ -827,7 +860,8 @@ def apply(fn: Callable[..., Any],
         (dict, FrozenDict)) and 'params' in variables['params']:
       raise errors.ApplyScopeInvalidVariablesStructureError(variables)
 
-    with bind(variables, rngs=rngs, mutable=mutable).temporary() as root:
+    with bind(variables, rngs=rngs, mutable=mutable,
+              flags=flags).temporary() as root:
       y = fn(root, *args, **kwargs)
     if mutable is not False:
       return y, root.mutable_variables()
@@ -838,12 +872,14 @@ def apply(fn: Callable[..., Any],
 
 
 def init(fn: Callable[..., Any],
-         mutable: CollectionFilter = True) -> Callable[..., Any]:
+         mutable: CollectionFilter = True,
+         flags: Optional[Dict] = None) -> Callable[..., Any]:
   """Functionalize a `Scope` function for initialization.
 
   Args:
     fn: a function taking a `Scope` as its first argument.
     mutable: the filter determining which variable collections are mutable.
+    flags: internal flags.
 
   Returns:
     `fn` with the scope partially applied.
@@ -857,7 +893,9 @@ def init(fn: Callable[..., Any],
                        '`jax.PRNGKey`.')
     if not isinstance(rngs, dict):
       rngs = {'params': rngs}
-    return apply(fn, mutable=mutable)({}, *args, rngs=rngs, **kwargs)
+    init_flags = {**(flags if flags is not None else {}), 'initializing': True}
+    return apply(fn, mutable=mutable, flags=init_flags)({}, *args, rngs=rngs,
+                                                        **kwargs)
 
   return wrapper
 
