@@ -538,6 +538,79 @@ class CallCompactUnboundModuleError(FlaxError):
   def __init__(self):
     super().__init__('Can\'t call compact methods on unbound modules')
 
+class CallSetupUnboundModuleError(FlaxError):
+  """
+  This error occurs when you are trying to call `.setup()` directly. For instance, the
+  error will be raised when trying to run this code::
+
+    from flax import linen as nn
+    import jax.numpy as jnp
+
+    class MyModule(nn.Module):
+      def setup(self):
+        self.submodule = MySubModule()
+
+    module = MyModule()
+    module.setup() # <-- ERROR!
+    submodule = module.submodule
+
+  In general you shouldn't call `.setup()` yourself, if you need to get access
+  to a field or submodule defined inside `setup` you can instead create a function
+  to extract it and pass it to `nn.apply`::
+
+    # setup() will be called automatically by `nn.apply`
+    def get_submodule(module):
+      return module.submodule.clone() # avoid leaking the Scope
+
+    empty_variables = {} # you can also use the real variables
+    submodule = nn.apply(get_submodule, module)(empty_variables)
+
+  """
+  def __init__(self):
+    super().__init__('Can\'t call compact methods on unbound modules')
+
+class InvalidInstanceModuleError(FlaxError):
+  """
+  This error occurs when you are trying to call `.init()`, `.init_with_output()`, `.apply() or `.bind()`
+  on the Module class itself, instead of an instance of the Module class.
+  For example, the error will be raised when trying to run this code::
+
+    class B(nn.Module):
+      @nn.compact
+      def __call__(self, x):
+        return x
+
+    k = random.PRNGKey(0)
+    x = random.uniform(random.PRNGKey(1), (2,))
+    B.init(k, x)   # B is module class, not B() a module instance
+    B.apply(vs, x)   # similar issue with apply called on class instead of instance.
+  """
+  def __init__(self):
+    super().__init__('Can only call init, init_with_output or apply methods on an instance of the Module class, not the Module class itself')
+
+class IncorrectPostInitOverrideError(FlaxError):
+  """
+  This error occurs when you overrode `.__post_init__()` without calling `super().__post_init__()`.
+  For example, the error will be raised when trying to run this code::
+
+    from flax import linen as nn
+    import jax.numpy as jnp
+    import jax
+    class A(nn.Module):
+      x: float
+      def __post_init__(self):
+        self.x_square = self.x ** 2
+        # super().__post_init__() <-- forgot to add this line
+      @nn.compact
+      def __call__(self, input):
+        return input + 3
+
+    r = A(x=3)
+    r.init(jax.random.PRNGKey(2), jnp.ones(3))
+  """
+  def __init__(self):
+    super().__init__('Overrode `.__post_init__()` without calling `super().__post_init__()`')
+
 
 class InvalidCheckpointError(FlaxError):
   """
@@ -551,8 +624,8 @@ class InvalidCheckpointError(FlaxError):
     super().__init__(f'Trying to save an outdated checkpoint at step: "{step}" and path: "{path}".')
 
 
-class GDACheckpointingRequiredError(FlaxError):
-  """To optimally save and restore a GlobalDeviceArray, use GlobalAsyncCheckpointManager.
+class MPACheckpointingRequiredError(FlaxError):
+  """To optimally save and restore a multiprocess array (GDA or jax Array outputted from pjit), use GlobalAsyncCheckpointManager.
 
   You can create an GlobalAsyncCheckpointManager at top-level and pass it as
   argument::
@@ -565,42 +638,62 @@ class GDACheckpointingRequiredError(FlaxError):
   def __init__(self, path, step):
     super().__init__(
         f'Checkpoint failed at step: "{step}" and path: "{path}": Target '
-        'contains a GlobalDeviceArray should be saved/restored with a '
+        'contains a multiprocess array should be saved/restored with a '
         'GlobalAsyncCheckpointManager.'
     )
 
 
-class GDARestoreTargetRequiredError(FlaxError):
-  """Provide a valid target when restoring a checkpoint with GlobalDeviceArray.
-
-  To restore a checkpoint that contains a GlobalDeviceArray, make sure
-  to pass a valid ``target`` argument that contains GDAs with correct global
-  meshes and partition specs.
+class MPARestoreTargetRequiredError(FlaxError):
+  """Provide a valid target when restoring a checkpoint with a multiprocess array.
 
+  Multiprocess arrays need a sharding (global meshes and partition specs) to be
+  initialized. Therefore, to restore a checkpoint that contains a multiprocess
+  array, make sure the ``target`` you passed contains valid multiprocess arrays
+  at the corresponding tree structure location. If you cannot provide a full
+  valid ``target``, consider ``allow_partial_mpa_restoration=True``.
   """
 
-  def __init__(self, path, step):
-    super().__init__(
+  def __init__(self, path, step, key=None):
+    error_msg = (
         f'Restore checkpoint failed at step: "{step}" and path: "{path}": '
-        'Checkpoints containing a GlobalDeviceArray need to be restored with '
-        'a valid target.'
-    )
+        'Checkpoints containing a multiprocess array need to be restored with '
+        'a target with pre-created arrays. If you cannot provide a full valid '
+        'target, consider ``allow_partial_mpa_restoration=True``. ')
+    if key:
+      error_msg += f'This error fired when trying to restore array at {key}.'
+    super().__init__(error_msg)
 
-class GDARestoreDataCorruptedError(FlaxError):
-  """A GDA stored in Google Cloud Storage doesn't contain
-  a "commit_success.txt" file, which should be written at the end of the save.
+class MPARestoreDataCorruptedError(FlaxError):
+  """A multiprocess array stored in Google Cloud Storage doesn't contain a "commit_success.txt" file, which should be written at the end of the save.
 
   Failure of finding it could indicate a corruption of your saved GDA data.
   """
 
   def __init__(self, step, path):
     super().__init__(
-        f'Restore checkpoint failed at step: "{step}" on GlobalDeviceArray at '
-        f' "{path}": No "commit_success.txt" found on this GDA directory. '
+        f'Restore checkpoint failed at step: "{step}" on multiprocess array at '
+        f' "{path}": No "commit_success.txt" found on this "_gda" directory. '
         'Was its save halted before completion?'
     )
 
 
+class MPARestoreTypeNotMatchError(FlaxError):
+  """Make sure the multiprocess array type you use matches your configuration in jax.config.jax_array.
+
+  If you turned `jax.config.jax_array` on, you should use
+  `jax.experimental.array.Array` everywhere, instead of using
+  `GlobalDeviceArray`. Otherwise, avoid using jax.experimental.array
+  to restore your checkpoint.
+  """
+
+  def __init__(self, step, gda_path):
+    super().__init__(
+        f'Restore checkpoint failed at step: "{step}" on multiprocess array at '
+        f' "{gda_path}": The array type provided by the target does not match '
+        'the JAX global configuration, namely the jax.config.jax_array.'
+    )
+
+
 #################################################
 # transforms.py errors                          #
 #################################################
@@ -633,7 +726,7 @@ class TransformTargetError(FlaxError):
       @nn.compact
       def __call__(self, x):
         return nn.vmap(
-            lambda mdl, x: mdl(x), 
+            lambda mdl, x: mdl(x),
             variable_axes={'params': 0}, split_rngs={'params': True})(nn.Dense(3), x)
 
   """
