@@ -18,17 +18,20 @@ import dataclasses
 import enum
 import functools
 import inspect
+import re
 import threading
 import typing
 import weakref
-from typing import (Any, Callable, Dict, Generic, Iterable, List, Optional,
-                    Sequence, Set, Tuple, Type, TypeVar, Union, overload)
+from typing import (Any, Callable, Dict, Iterable, List, NamedTuple, Mapping,
+                    Optional, Set, Tuple, Type, TypeVar, Union, overload)
 
 import jax
 import numpy as np
+import jax.numpy as jnp
 from typing_extensions import \
     dataclass_transform  # pytype: disable=not-supported-yet
 
+import flax
 from flax import (config, core, errors, serialization, traceback_util,
                   traverse_util)
 from flax.core import Scope
@@ -36,9 +39,9 @@ from flax.core.frozen_dict import FrozenDict
 from flax.core.scope import (  # pylint: disable=g-multiple-import
     CollectionFilter, DenyList, FrozenVariableDict, Variable, VariableDict,
     union_filters)
+from flax.ids import FlaxId
 from flax.ids import uuid
-from flax.linen import summary
-
+from flax.linen import kw_only_dataclasses
 
 
 traceback_util.register_exclusion(__file__)
@@ -61,6 +64,16 @@ TestScope = type('TestScope',
 
 # pylint: disable=protected-access,attribute-defined-outside-init
 
+def _get_value_representation(x: Any) -> 'flax.linen.summary._ValueRepresentation':
+  from flax.linen import summary
+
+  if isinstance(x, (int, float, bool, type(None))) or (
+    isinstance(x, np.ndarray) and np.isscalar(x)):
+    return summary._ObjectRepresentation(x)
+  try:
+    return summary._ArrayRepresentation(jnp.shape(x), jnp.result_type(x))
+  except:
+    return summary._ObjectRepresentation(x)
 
 def _indent(x: str, num_spaces: int):
   indent_str = ' ' * num_spaces
@@ -104,36 +117,73 @@ def _module_repr(module: 'Module', num_spaces: int = 4):
   else:
     return f'{cls_name}()'
 
+# Tabulation utilities.
+# -----------------------------------------------------------------------------
+
+_find_non_lifted_module = re.compile(r'.*\((.*)\)')
+
+def _fix_path_part(part: str):
+  """Fixes a path part by removing transformation name and parenthesis sometimes
+  inserted by lifted transformations"""
+  match = _find_non_lifted_module.match(part)
+  if match:
+    return match.group(1)
+  return part
+
+@dataclasses.dataclass
+class _CallInfo:
+  index: int
+  path: Tuple[str, ...]
+  module_type: Type['Module']
+  method: str
+  args: Tuple[Any, ...]
+  kwargs: Dict[str, Any]
+  outputs: Any
+
+@dataclasses.dataclass
+class _CallInfoContext(threading.local):
+  index: int
+  calls: List[_CallInfo]
+
+  def get_call_index(self, module: 'Module') -> int:
+    index = self.index
+    self.index += 1
+    return index
+
+@contextlib.contextmanager
+def _tabulate_context():
+  _context.call_info_stack.append(_CallInfoContext(0, []))
+  try:
+    yield
+  finally:
+    _context.call_info_stack.pop()
 
 # Track parent relationship across Modules.
 # -----------------------------------------------------------------------------
-class _DynamicContext:
+class _DynamicContext(threading.local):
   """Dynamic context."""
   # TODO(marcvanzee): switch to using contextvars once minimum python version is
   # 3.7
 
   def __init__(self):
-    self._thread_data = threading.local()
-
-  @property
-  def module_stack(self):
-    if not hasattr(self._thread_data, 'module_stack'):
-      self._thread_data.module_stack = [None,]
-    return self._thread_data.module_stack
-
-  @property
-  def capture_stack(self):
-    """Keeps track of the active capture_intermediates filter functions."""
-    if not hasattr(self._thread_data, 'capture_stack'):
-      self._thread_data.capture_stack = []
-    return self._thread_data.capture_stack
+    self.module_stack = [None,]
+    self.capture_stack = []
+    self.call_info_stack = []
 
 # The global context
 _context = _DynamicContext()
 
 
 class _Sentinel:
-  pass
+
+  def __copy__(self):
+    return self  # Do not copy singleton sentinel.
+
+  def __deepcopy__(self, memo):
+    del memo
+    return self  # Do not copy singleton sentinel.
+
+
 _unspecified_parent = _Sentinel()
 
 
@@ -467,7 +517,7 @@ _UNDEFINED_COPY_PICKLE_METHODS = (
     '__reduce__', '__reduce_ex__', '__copy__', '__deepcopy__')
 
 
-_caches: 'weakref.WeakKeyDictionary[Scope, Dict[int, Module]]' = (
+_caches: 'weakref.WeakKeyDictionary[Scope, weakref.WeakValueDictionary[FlaxId, Module]]' = (
     weakref.WeakKeyDictionary())
 
 
@@ -478,6 +528,33 @@ tuple_init = lambda: ()
 capture_call_intermediates = lambda _, method_name: method_name == '__call__'
 
 
+_ParentType = Union[Type['Module'], Type[Scope], Type[_Sentinel], None]
+
+
+class ParentDescriptor:
+  """Wraps parent module references in weak refs.
+
+  This prevents reference cycles from forming via parent links which can lead
+  to accidental OOMs in eager mode due to slow garbage collection as well as
+  spurious tracer leaks during jit compilation.
+
+  Note: "descriptors" are the underlying python mechanism for implementing
+  dynamic @property decorators.  We need to use a raw descriptor instead of the
+  more common decorator in order to force that the appropriate getter/setter
+  logic applies in subclasses even after various dataclass transforms.
+  """
+  def __get__(self, obj, objtype=None):
+    # check if obj is None, happens during %autoreload
+    if obj is None:
+      return None
+    parent = object.__getattribute__(obj, "_parent_ref")
+    return parent() if isinstance(parent, weakref.ReferenceType) else parent
+
+  def __set__(self, obj, value):
+    maybe_weak = weakref.ref(value) if isinstance(value, Module) else value
+    object.__setattr__(obj, "_parent_ref", maybe_weak)
+
+
 # Base Module definition.
 # -----------------------------------------------------------------------------
 
@@ -543,37 +620,25 @@ class Module:
     # Set empty class defaults.
     cls._state = _uninitialized_module_internal_state
     cls.scope: Optional[Scope] = None
+    # Handles weak referencing of parent Modules to prevent reference cycles.
+    cls._parent_ref = None
+    cls.parent = ParentDescriptor()
 
   @classmethod
   def _customized_dataclass_transform(cls):
-    """Handles final optional dataclass attributes: `parent` and `name`."""
-    # Use cls.__dict__ to get annotations of cls itself (no parent class).
+    """Transforms `cls` into a dataclass, with custom additional behavior.
+
+    1. Inject `parent` and `name` fields.  (If they are already present,
+       then check that they have the expected types.)
+    2. Set compare, hash, and repr to False for non-init fields.
+    3. Generate a hash function (if not provided by cls).
+    """
+    # Check reserved attributes have expected type annotations.
     annotations = dict(cls.__dict__.get('__annotations__', {}))
-    parent_annotation = Union[Type[Module], Type[Scope],
-                              Type[_Sentinel], None]
-    if ('parent' in annotations
-        and annotations['parent'] != parent_annotation):
+    if annotations.get('parent', _ParentType) != _ParentType:
       raise errors.ReservedModuleAttributeError(annotations)
-    if 'name' in annotations and annotations['name'] not in ('str', str):
+    if annotations.get('name', str) not in ('str', str, Optional[str]):
       raise errors.ReservedModuleAttributeError(annotations)
-    # Add `parent` and `name` default fields at end.
-    # We temporarily modify base class __dataclass_fields__ to force desired
-    # argument behavior and ordering from dataclass class-transform.
-    parent_dataclass_fields = []
-    for clz in cls.__mro__[1:]:
-      pdf = dict(getattr(clz, '__dataclass_fields__', {}))
-      parent_dataclass_fields.append(pdf)
-
-      # Remove 'parent' and 'name' from parents because we always want parent
-      # and name to show up last in the dataclass args.
-      if 'parent' in pdf:
-        clz.__dataclass_fields__.pop('parent')  # pytype: disable=attribute-error
-      if 'name' in pdf:
-        clz.__dataclass_fields__.pop('name')  # pytype: disable=attribute-error
-
-    annotations['parent'] = parent_annotation
-    cls.parent = dataclasses.field(repr=False, default=_unspecified_parent)
-    annotations['name'] = str
 
     # any non-init field will only be set in setup
     # During __hash__ and __eq__ the field is not set yet
@@ -585,17 +650,21 @@ class Module:
         field_meta.hash = False
         field_meta.repr = False
 
-    cls.name = None  # default value of name is None.
-    cls.__annotations__ = annotations
+    extra_fields = [('parent', _ParentType,
+                     kw_only_dataclasses.field(
+                         repr=False, default=_unspecified_parent,
+                         kw_only=True)),
+                    ('name', Optional[str],
+                     kw_only_dataclasses.field(default=None, kw_only=True))]
+
     # Now apply dataclass transform (which operates in-place).
     # Do generate a hash function only if not provided by the class.
-    dataclasses.dataclass(
-        cls, unsafe_hash='__hash__' not in cls.__dict__, repr=False)  # pytype: disable=wrong-keyword-args
+    kw_only_dataclasses.dataclass(
+        cls,
+        unsafe_hash='__hash__' not in cls.__dict__,
+        repr=False,
+        extra_fields=extra_fields)  # pytype: disable=wrong-keyword-args
     cls.__hash__ = _wrap_hash(cls.__hash__)
-    # Restore original base class __dataclass_fields__.
-    for clz, pdf in zip(cls.__mro__[1:], parent_dataclass_fields):
-      if dataclasses.is_dataclass(clz):
-        clz.__dataclass_fields__ = pdf
 
   @classmethod
   def _verify_single_or_no_compact(cls):
@@ -638,8 +707,11 @@ class Module:
     is_compact_method = hasattr(fun, 'compact')
     fun_name = getattr(fun, '__name__', 'unnamed_function')
     is_setup_method = fun_name == 'setup'
+    add_call_info = not is_setup_method and len(_context.call_info_stack) > 0
     # We lazily call setup() only when needed.
     if is_setup_method:
+      if self.scope is None:
+        raise errors.CallSetupUnboundModuleError()
       is_recurrent = self._state.in_setup
       self._state.in_setup = True
     else:
@@ -652,15 +724,27 @@ class Module:
       self._state.in_compact_method = True
     _context.module_stack.append(self)
     try:
+      # get call info
+      if add_call_info:
+        call_index = _context.call_info_stack[-1].get_call_index(self)
+        scope_path = jax.tree_util.tree_map(_fix_path_part, self.scope.path)
+
+      # call method
       if _use_named_call:
         with jax.named_scope(_derive_profiling_name(self, fun)):
           y = fun(self, *args, **kwargs)
       else:
         y = fun(self, *args, **kwargs)
+
       if _context.capture_stack:
         filter_fn = _context.capture_stack[-1]
         if filter_fn and filter_fn(self, fun_name):
           self.sow('intermediates', fun_name, y)
+      if add_call_info:
+        _args, _kwargs, _y = jax.tree_util.tree_map(
+          _get_value_representation, (args, kwargs, y), is_leaf=lambda x: x is None)
+        _context.call_info_stack[-1].calls.append(
+          _CallInfo(call_index, scope_path, type(self), fun.__name__, _args, _kwargs, _y))
       return y
     finally:
       _context.module_stack.pop()
@@ -722,8 +806,11 @@ class Module:
     if name in self.__dict__:
       return self.__dict__[name]
     else:
-      raise AttributeError(
-          f'"{self.__class__.__name__}" object has no attribute "{name}"')
+      msg = f'"{self.__class__.__name__}" object has no attribute "{name}".'
+      if self.scope is None:
+        msg += (f' If "{name}" is defined in \'.setup()\', remember these fields '
+          'are only accessible from inside \'init\' or \'apply\'.')
+      raise AttributeError(msg)
 
   def __dir__(self) -> List[str]:
     """Call setup() before listing attributes."""
@@ -827,7 +914,7 @@ class Module:
     """Registers a submodule."""
     assert self.scope, 'Trying to register submodules on unbound scope.'
     root = self.scope.root
-    cache = _caches.get(root, {})
+    cache = _caches.get(root, weakref.WeakValueDictionary())
     _caches[root] = cache
     queue = []
     def adopt_attr_modules(cache, queue, suffix, subvalue):
@@ -839,9 +926,12 @@ class Module:
           # Preserve sharing-by-reference relationships during adoption
           # via cache keyed on unique instance ids.
           key = subvalue._id
-          if key not in cache:
-            cache[key] = subvalue.clone()
-          subvalue = cache[key]
+          if key in cache:
+            subvalue = cache[key]
+          else:
+            # We must bind to local variable before adding to weakvalue dict.
+            subvalue = subvalue.clone()
+            cache[key] = subvalue
         if subvalue.name is None:
           object.__setattr__(subvalue, 'parent', self)
           object.__setattr__(subvalue, 'name', f'{name}{suffix}')
@@ -1058,6 +1148,16 @@ class Module:
       raise ValueError("Can't check if running under init() on unbound modules")
     return self.scope.get_flag('initializing', False)
 
+  def _module_checks(self):
+    """Run standard runtime checks."""
+
+    if not isinstance(self, Module):
+      raise errors.InvalidInstanceModuleError()
+
+    overridden_post_init = self.__post_init__ != Module.__post_init__
+    if overridden_post_init and not hasattr(self, "_id"):
+      raise errors.IncorrectPostInitOverrideError()
+
   @traceback_util.api_boundary
   def bind(self,
            variables: VariableDict,
@@ -1113,6 +1213,8 @@ class Module:
     Returns:
       A copy of this instance with bound variables and RNGs.
     """
+    Module._module_checks(self)
+
     del args
     scope = core.bind(variables, rngs=rngs, mutable=mutable)
     return self.clone(parent=scope)
@@ -1177,6 +1279,8 @@ class Module:
       mutable, returns ``(output, vars)``, where ``vars`` are is a dict
       of the modified collections.
     """
+    Module._module_checks(self)
+
     if method is None:
       method = self.__call__
     method = _get_unbound_fn(method)
@@ -1217,6 +1321,8 @@ class Module:
       `(output, vars)``, where ``vars`` are is a dict of the modified
       collections.
     """
+    Module._module_checks(self)
+
     if not isinstance(rngs, dict):
       if not core.scope._is_valid_rng(rngs):
         raise errors.InvalidRngError(
@@ -1270,6 +1376,8 @@ class Module:
     Returns:
       The initialized variable dict.
     """
+    Module._module_checks(self)
+
     _, v_out = self.init_with_output(
         rngs,
         *args,
@@ -1406,21 +1514,65 @@ class Module:
     self.scope.put_variable(col, name, xs)
     return True
 
+  def perturb(self, name: str, value: T, collection: str = 'perturbations') -> T:
+    """Add an zero-value variable ('perturbation') to the intermediate value.
+
+    The gradient of `value` would be the same as the gradient of this
+    perturbation variable. Therefore, if you define your loss function with
+    both params and perturbations as standalone arguments, you can get the
+    intermediate gradients of `value` by running `jax.grad` on the perturbation
+    argument.
+
+    Note: this is an experimental API and may be tweaked later for better
+    performance and usability.
+    At its current stage, it creates extra dummy variables that occupies extra
+    memory space. Use it only to debug gradients in training.
+
+    Example::
+
+      import jax
+      import jax.numpy as jnp
+      import flax.linen as nn
+
+      class Foo(nn.Module):
+          @nn.compact
+          def __call__(self, x):
+              x = nn.Dense(3)(x)
+              x = self.perturb('dense3', x)
+              return nn.Dense(2)(x)
+
+      def loss(params, perturbations, inputs, targets):
+        variables = {'params': params, 'perturbations': perturbations}
+        preds = model.apply(variables, inputs)
+        return jnp.square(preds - targets).mean()
+
+      x = jnp.ones((2, 9))
+      y = jnp.ones((2, 2))
+      model = Foo()
+      variables = model.init(jax.random.PRNGKey(0), x)
+      intm_grads = jax.grad(loss, argnums=1)(variables['params'], variables['perturbations'], x, y)
+      print(intm_grads['dense3']) # ==> [[-1.456924   -0.44332537  0.02422847]
+                                  #      [-1.456924   -0.44332537  0.02422847]]
+
+    """
+    value += self.variable(collection, name, lambda: jnp.zeros_like(value)).value
+    return value
+
   def tabulate(
     self,
     rngs: Union[PRNGKey, RNGSequences],
     *args,
-    method: Optional[Callable[..., Any]] = None,
-    mutable: CollectionFilter = True,
     depth: Optional[int] = None,
-    exclude_methods: Sequence[str] = (),
+    show_repeated: bool = False,
+    mutable: CollectionFilter = True,
+    console_kwargs: Optional[Mapping[str, Any]] = None,
     **kwargs) -> str:
     """Creates a summary of the Module represented as a table.
 
-    This method has the same signature as `init`, but instead of returning
-    the variables, it returns the string summarizing the Module in a table.
-    `tabulate` uses `jax.eval_shape` to run the forward computation without
-    consuming any FLOPs or allocating memory.
+    This method has the same signature and internally calls `Module.init`,
+    but instead of returning the variables, it returns the string summarizing
+    the Module in a table. `tabulate` uses `jax.eval_shape` to run the forward
+    computation without consuming any FLOPs or allocating memory.
 
     Example::
 
@@ -1441,61 +1593,60 @@ class Module:
 
     This gives the following output::
 
-                         Foo Summary
-      ┏━━━━━━━━━┳━━━━━━━━━━━━━━━┳━━━━━━━━━━━━━━━━━━━━━━┓
-      ┃ path    ┃ outputs       ┃ params               ┃
-      ┡━━━━━━━━━╇━━━━━━━━━━━━━━━╇━━━━━━━━━━━━━━━━━━━━━━┩
-      │ Inputs  │ float32[16,9] │                      │
-      ├─────────┼───────────────┼──────────────────────┤
-      │ Dense_0 │ float32[16,4] │ bias: float32[4]     │
-      │         │               │ kernel: float32[9,4] │
-      │         │               │                      │
-      │         │               │ 40 (160 B)           │
-      ├─────────┼───────────────┼──────────────────────┤
-      │ Dense_1 │ float32[16,2] │ bias: float32[2]     │
-      │         │               │ kernel: float32[4,2] │
-      │         │               │                      │
-      │         │               │ 10 (40 B)            │
-      ├─────────┼───────────────┼──────────────────────┤
-      │ Foo     │ float32[16,2] │                      │
-      ├─────────┼───────────────┼──────────────────────┤
-      │         │         Total │ 50 (200 B)           │
-      └─────────┴───────────────┴──────────────────────┘
+                                      Foo Summary
+      ┏━━━━━━━━━┳━━━━━━━━┳━━━━━━━━━━━━━━━┳━━━━━━━━━━━━━━━┳━━━━━━━━━━━━━━━━━━━━━━┓
+      ┃ path    ┃ module ┃ inputs        ┃ outputs       ┃ params               ┃
+      ┡━━━━━━━━━╇━━━━━━━━╇━━━━━━━━━━━━━━━╇━━━━━━━━━━━━━━━╇━━━━━━━━━━━━━━━━━━━━━━┩
+      │         │ Foo    │ float32[16,9] │ float32[16,2] │                      │
+      ├─────────┼────────┼───────────────┼───────────────┼──────────────────────┤
+      │ Dense_0 │ Dense  │ float32[16,9] │ float32[16,4] │ bias: float32[4]     │
+      │         │        │               │               │ kernel: float32[9,4] │
+      │         │        │               │               │                      │
+      │         │        │               │               │ 40 (160 B)           │
+      ├─────────┼────────┼───────────────┼───────────────┼──────────────────────┤
+      │ Dense_1 │ Dense  │ float32[16,4] │ float32[16,2] │ bias: float32[2]     │
+      │         │        │               │               │ kernel: float32[4,2] │
+      │         │        │               │               │                      │
+      │         │        │               │               │ 10 (40 B)            │
+      ├─────────┼────────┼───────────────┼───────────────┼──────────────────────┤
+      │         │        │               │         Total │ 50 (200 B)           │
+      └─────────┴────────┴───────────────┴───────────────┴──────────────────────┘
 
-                Total Parameters: 50 (200 B)
+                            Total Parameters: 50 (200 B)
 
     **Note**: rows order in the table does not represent execution order,
     instead it aligns with the order of keys in `variables` which are sorted
     alphabetically.
 
     Args:
-      rngs: The rngs for the variable collections.
+      rngs: The rngs for the variable collections as passed to `Module.init`.
       *args: The arguments to the forward computation.
-      method: An optional method. If provided, applies this method. If not
-        provided, applies the ``__call__`` method.
-      mutable: Can be bool, str, or list. Specifies which collections should be
-        treated as mutable: ``bool``: all/no collections are mutable.
-        ``str``: The name of a single mutable collection. ``list``: A
-        list of names of mutable collections. By default all collections
-        except 'intermediates' are mutable.
       depth: controls how many submodule deep the summary can go. By default its
         `None` which means no limit. If a submodule is not shown because of the
-        depth limit, its parameter count and bytes will be added to the row of
-        its first shown ancestor such that the sum of all rows always adds up to
-        the total number of parameters of the Module.
-      exclude_methods: A sequence of strings that specifies which methods should
-        be ignored. In case a module calls a helper method from its main method,
-        use this argument to exclude the helper method from the summary to avoid
-        ambiguity.
+        depth limit, its parameter count and bytes will be added to the row of its
+        first shown ancestor such that the sum of all rows always adds up to the
+        total number of parameters of the Module.
+      show_repeated: If `True`, repeated calls to the same module will be shown
+        in the table, otherwise only the first call will be shown. Default is
+        `False`.
+      mutable: Can be bool, str, or list. Specifies which collections should be
+        treated as mutable: ``bool``: all/no collections are mutable. ``str``: The
+        name of a single mutable collection. ``list``: A list of names of mutable
+        collections. By default all collections except 'intermediates' are
+        mutable.
+      console_kwargs: An optional dictionary with additional keyword arguments that
+        are passed to `rich.console.Console` when rendering the table. Default arguments
+        are `{'force_terminal': True, 'force_jupyter': False}`.
       **kwargs: keyword arguments to pass to the forward computation.
 
     Returns:
       A string summarizing the Module.
     """
+    from flax.linen import summary
 
-    tabulate_fn = summary.tabulate(self, rngs, method=method,
-                                   mutable=mutable, depth=depth,
-                                   exclude_methods=exclude_methods)
+    tabulate_fn = summary.tabulate(self, rngs, depth=depth,
+                                   show_repeated=show_repeated, mutable=mutable,
+                                   console_kwargs=console_kwargs)
     return tabulate_fn(*args, **kwargs)
 
 
