@@ -224,8 +224,8 @@ class BatchNorm(Module):
     """Normalizes the input using batch statistics.
 
     NOTE:
-    During initialization (when parameters are mutable) the running average
-    of the batch statistics will not be updated. Therefore, the inputs
+    During initialization (when `self.is_initializing()` is `True`) the running
+    average of the batch statistics will not be updated. Therefore, the inputs
     fed during initialization don't need to match that of the actual input
     distribution and the reduction axis (set with `axis_name`) does not have
     to exist.
@@ -245,9 +245,6 @@ class BatchNorm(Module):
     reduction_axes = tuple(i for i in range(x.ndim) if i not in feature_axes)
     feature_shape = [x.shape[ax] for ax in feature_axes]
 
-    # see NOTE above on initialization behavior
-    initializing = self.is_mutable_collection('params')
-
     ra_mean = self.variable('batch_stats', 'mean',
                             lambda s: jnp.zeros(s, jnp.float32),
                             feature_shape)
@@ -261,10 +258,10 @@ class BatchNorm(Module):
       mean, var = _compute_stats(
           x, reduction_axes,
           dtype=self.dtype,
-          axis_name=self.axis_name if not initializing else None,
+          axis_name=self.axis_name if not self.is_initializing() else None,
           axis_index_groups=self.axis_index_groups)
 
-      if not initializing:
+      if not self.is_initializing():
         ra_mean.value = self.momentum * ra_mean.value + (1 -
                                                          self.momentum) * mean
         ra_var.value = self.momentum * ra_var.value + (1 - self.momentum) * var
@@ -296,6 +293,15 @@ class LayerNorm(Module):
     scale_init: Initializer for scale, by default, one.
     reduction_axes: Axes for computing normalization statistics.
     feature_axes: Feature axes for learned bias and scaling.
+    axis_name: the axis name used to combine batch statistics from multiple
+      devices. See `jax.pmap` for a description of axis names (default: None).
+      This is only needed if the model is subdivided across devices, i.e. the
+      array being normalized is sharded across devices within a pmap.
+    axis_index_groups: groups of axis indices within that named axis
+      representing subsets of devices to reduce over (default: None). For
+      example, `[[0, 1], [2, 3]]` would independently batch-normalize over
+      the examples on the first two and last two devices. See `jax.lax.psum`
+      for more details.
   """
   epsilon: float = 1e-6
   dtype: Optional[Dtype] = None
@@ -306,6 +312,8 @@ class LayerNorm(Module):
   scale_init: Callable[[PRNGKey, Shape, Dtype], Array] = initializers.ones
   reduction_axes: Axes = -1
   feature_axes: Axes = -1
+  axis_name: Optional[str] = None
+  axis_index_groups: Any = None
 
   @compact
   def __call__(self, x):
@@ -317,8 +325,8 @@ class LayerNorm(Module):
     Returns:
       Normalized inputs (the same shape as inputs).
     """
-    # TODO(jheek) suport axis_name for model parallelism?
-    mean, var = _compute_stats(x, self.reduction_axes, self.dtype, None, None)
+    mean, var = _compute_stats(x, self.reduction_axes, self.dtype,
+                               self.axis_name, self.axis_index_groups)
 
     return _normalize(
         self, x, mean, var, self.reduction_axes, self.feature_axes,
@@ -350,6 +358,15 @@ class GroupNorm(Module):
         be done by the next layer.
       bias_init: Initializer for bias, by default, zero.
       scale_init: Initializer for scale, by default, one.
+      axis_name: the axis name used to combine batch statistics from multiple
+        devices. See `jax.pmap` for a description of axis names (default: None).
+        This is only needed if the model is subdivided across devices, i.e. the
+        array being normalized is sharded across devices within a pmap.
+      axis_index_groups: groups of axis indices within that named axis
+        representing subsets of devices to reduce over (default: None). For
+        example, `[[0, 1], [2, 3]]` would independently batch-normalize over
+        the examples on the first two and last two devices. See `jax.lax.psum`
+        for more details.
   """
   num_groups: Optional[int] = 32
   group_size: Optional[int] = None
@@ -360,6 +377,8 @@ class GroupNorm(Module):
   use_scale: bool = True
   bias_init: Callable[[PRNGKey, Shape, Dtype], Array] = initializers.zeros
   scale_init: Callable[[PRNGKey, Shape, Dtype], Array] = initializers.ones
+  axis_name: Optional[str] = None
+  axis_index_groups: Any = None
 
   @compact
   def __call__(self, x):
@@ -405,9 +424,9 @@ class GroupNorm(Module):
                               (x.shape[0], num_groups, group_size))
       return stat.reshape((x.shape[0], num_groups * group_size))
 
-    # TODO(jheek): suport axis_name for model parallelism?
     mean, var = _compute_stats(
-        x.reshape(group_shape), reduction_axes, self.dtype, None, None)
+        x.reshape(group_shape), reduction_axes, self.dtype, self.axis_name,
+        self.axis_index_groups)
     mean = broadcast_stat(mean)
     var = broadcast_stat(var)
 
