@@ -433,9 +433,11 @@ def vmap(target: Target,
          variable_axes: Mapping[lift.CollectionFilter,
                                 lift.InOutAxis] = FrozenDict(),
          split_rngs: Mapping[lift.PRNGSequenceFilter, bool] = FrozenDict(),
-         in_axes=0, out_axes=0,
+         in_axes=0,
+         out_axes=0,
          axis_size: Optional[int] = None,
          axis_name: Optional[str] = None,
+         spmd_axis_name: Optional[str] = None,
          methods=None) -> Target:
   """A lifted version of ``jax.vmap``.
 
@@ -486,6 +488,9 @@ def vmap(target: Target,
       with parallel reduction primitives (e.g. `jax.lax.pmean`,
       `jax.lax.ppermute`, etc.)
     methods: If `target` is a `Module`, the methods of `Module` to vmap over.
+    spmd_axis_name: Axis name added to any pjit sharding constraints appearing
+      in `fn`. See also
+      https://github.com/google/flax/blob/main/flax/linen/partitioning.py.
 
   Returns:
     A batched/vectorized version of ``target``, with the same arguments but with
@@ -493,10 +498,16 @@ def vmap(target: Target,
     but with extra axes at positions indicated by ``out_axes``.
   """
   return lift_transform(
-      lift.vmap, target, variable_axes, split_rngs,
+      lift.vmap,
+      target,
+      variable_axes,
+      split_rngs,
       methods=methods,
-      in_axes=in_axes, out_axes=out_axes,
-      axis_size=axis_size, axis_name=axis_name)
+      in_axes=in_axes,
+      out_axes=out_axes,
+      axis_size=axis_size,
+      axis_name=axis_name,
+      spmd_axis_name=spmd_axis_name)
 
 
 def jit(target: Target,
@@ -561,6 +572,7 @@ def checkpoint(target: Target,
                rngs: lift.PRNGSequenceFilter = True,
                concrete: bool = False,
                prevent_cse: bool = True,
+               static_argnums: Union[int, Tuple[int, ...]] = (),
                policy: Optional[Callable[..., bool]] = None,
                methods=None) -> Target:
   """Lifted version of ``jax.checkpoint``.
@@ -588,6 +600,10 @@ def checkpoint(target: Target,
       ``pmap``, CSE can defeat the purpose of this decorator. But in some
       settings, like when used inside a ``scan``, this CSE prevention mechanism
       is unnecessary, in which case ``prevent_cse`` should be set to False.
+    static_argnums: Optional, int or sequence of ints, indicates which argument
+      values on which to specialize for tracing and caching purposes. Specifying
+      arguments as static can avoid ConcretizationTypeErrors when tracing, but
+      at the cost of more retracing overheads.
     policy: Experimental checkpoint policy, see ``jax.checkpoint``.
     methods: If `target` is a `Module`, the methods of `Module` to checkpoint.
 
@@ -595,9 +611,13 @@ def checkpoint(target: Target,
     A wrapped version of ``target``. When computing gradients intermediate
     computations will be re-computed on the backward pass.
   """
+  # subtract 1 from each static_argnums because 'self' is not passed to the
+  # lifted function
+  static_argnums = jax.tree_util.tree_map(lambda x: x - 1, static_argnums)
   return lift_transform(
       lift.checkpoint, target,
       variables=variables, rngs=rngs, concrete=concrete,
+      static_argnums=static_argnums,
       prevent_cse=prevent_cse, policy=policy,
       methods=methods)
 
@@ -865,16 +885,16 @@ def vjp(
 
     class LearnScale(nn.Module):
       @nn.compact
-      def __call__(self, x):
+      def __call__(self, x, y):
         p = self.param('scale', nn.initializers.zeros, ())
-        return p * x
+        return p * x * y
 
     class Foo(nn.Module):
       @nn.compact
-      def __call__(self, x):
-        y, bwd = nn.vjp(lambda mdl, x: mdl(x), LearnScale(), x)
-        params_grad, x_grad = bwd(jnp.ones(y.shape))
-        return y, params_grad, x_grad
+      def __call__(self, x, y):
+        z, bwd = nn.vjp(lambda mdl, x, y: mdl(x, y), LearnScale(), x, y)
+        params_grad, x_grad, y_grad = bwd(jnp.ones(z.shape))
+        return z, params_grad, x_grad, y_grad
 
   Args:
     fn: Function to be differentiated. Its arguments should be arrays, scalars,
@@ -1178,7 +1198,7 @@ def switch(
           nn.Sequential([nn.Dense(11), nn.Dense(5)]),
           nn.Dense(5),
         ]
-      
+
       @nn.compact
       def __call__(self, x, index):
         def head_fn(i):
@@ -1189,7 +1209,7 @@ def switch(
         if self.is_mutable_collection('params'):
           for branch in branches:
             _ = branch(self, x)
-          
+
         return nn.switch(index, branches, self, x)
 
   Args:
@@ -1238,7 +1258,7 @@ def custom_vjp(fn: Callable[..., Any],
   passed to `backward_fn`.
 
   The `backward_fn` receives the nondiff arguments, residuals, and the output
-  tangents. It should return a tuple containing the input and variable tangents.
+  tangents. It should return a tuple containing the variable and input tangents.
 
   Note that the vjp function returned by `nn.vjp` can be passed as residual and
   used in the `backward_fn`. The scope is unavailable during the backward pass.
@@ -1257,9 +1277,9 @@ def custom_vjp(fn: Callable[..., Any],
           return nn.vjp(f, mdl, x)
 
         def bwd(vjp_fn, y_t):
-          input_t, params_t = vjp_fn(y_t)
+          params_t, *inputs_t = vjp_fn(y_t)
           params_t = jax.tree_util.tree_map(jnp.sign, params_t)
-          return input_t, params_t
+          return (params_t, *inputs_t)
 
         sign_grad = nn.custom_vjp(
             f, forward_fn=fwd, backward_fn=bwd)
@@ -1276,9 +1296,9 @@ def custom_vjp(fn: Callable[..., Any],
       ``backward_fn``.
     backward_fn: arguments are passed as
       ``(*nondiff_args, residuals, tangents)`` The function should return a
-      tuple containing the tangents for the input arguments (except the module
-      and nondiff args) and the variable tangents for the collections specified
-      by `grad_vars`.
+      tuple containing the tangents for the variable in the collections
+      specified by `grad_vars` and the input arguments (except the module and
+      nondiff args).
     grad_vars: The collections for which a vjp will be computed
       (default: "params").
     nondiff_argnums: arguments for which no vjp is computed.
